/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:      Wioletta.Kozlowska@cern.ch
 * based on the T.Boehlen version from 2011
 * Last change: 16-Dec-2016
 */
#ifndef __DATA_EVENTBIN_H
#define __DATA_EVENTBIN_H

#include "usrbin.h"
#include "tool/fortran.h"

#define FTNVGET(V,T,B)	{V=*(T*)B; B+=sizeof(T);}
//TTB: return V=Variable, T=Type, B=buffer,  return S=String, L=Length

class Eventbin {
private:
	int		_detector;	// detector loaded
	Usrbin::Type	_type;		// type of eventbin
	int		_score;		// quantity scored
public:
	int		nx, ny, nz;
	double		xlow, ylow, zlow;
	double		xhigh, yhigh, zhigh;
	double		dx, dy, dz;
	double		xofs,  yofs,  zofs;	// offset of usrbin
	double		x0,  y0;		// for R-Z-Phi binning
	double		weight;
	int		nhits;

private:
	FortranFile	file;
	std::string	filename;

public:
	Eventbin() {};
	Eventbin(const std::string& fn, int det=0) { load(fn,det); };
	~Eventbin() {cleanup();};

	void cleanup();
	int load(const std::string& fn, int det=0);

	/* get/set */
	int	detector()	const	{ return _detector; }
	Usrbin::Type type()	const	{ return _type; }
	int	score()		const	{ return _score; }

	int eventbini(const double x) const {return int((x-xlow)/dx);}
	int eventbinj(const double y) const {return int((y-ylow)/dy);}
	int eventbink(const double z) const {return int((z-zlow)/dz);}

	int readHeader();
	char* readEvent(char* buffer, unsigned sizeBuffer);
}; // class Eventbin

#endif
