////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BXDF_MICROFACET_H
#define BXDF_MICROFACET_H


// Torrance sparrow microfacet model

#include "bxdfbase.h"
#include "bxdffresnel.h"
#include "microfacetdistribution.h"

class CBxDFBase;
class CFresnel;
class CMicrofacetDistribution;


class CBxDFMicrofacet :  public CBxDFBase
{
public:
  // Microfacet Public Methods
  CBxDFMicrofacet( CFresnel* pFresnel, CMicrofacetDistribution* pMicroDist );

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	virtual Color f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
  virtual Color Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;
  virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const override;

  virtual float G(const Vector& Out, const Vector& In, const Vector& wh, const Vector& Normal) const {
      return m_pMicroDist->G( Out, In, wh, Normal );
  }

private:
  CFresnel*                m_pFresnel;
	CMicrofacetDistribution* m_pMicroDist;

};

#endif
// vim:ts=2
