////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MAT_PRIMER_H
#define MAT_PRIMER_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "baseintegrator.h"
#include "bxdflafortune.h"

class CBxDFLaFortune;

class CMatPrimer: public CPhysMatInterface
{
public:
  CMatPrimer( CBaseIntegrator* pInt );
  virtual ~CMatPrimer(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual Color f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0f ) override;

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual Color Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;

protected:
  CBxDFLaFortune*         m_pBxDFDiffuse;
};

#endif
// vim:ts=2
