////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matskin.h"

CMatSkin::CMatSkin(CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ), false /* false = no MC sampling required */ )
{
  static Color Diffuse( 0.428425,  0.301341,  0.331054 );
	static float xy0[3] =     { -1.131747f, -1.016939f, -0.966018f};
	static float z0[3] =      { -1.209182f, -1.462488f, -1.222419f};
	static float e0[3] =      {  6.421658f,  3.699932f,  3.524889f};
	static float xy1[3] =     { -0.546570f, -0.643533f, -0.638934f};
	static float z1[3] =      {  0.380123f,  0.410559f,  0.437367f};
	static float e1[3] =      {  3.685044f,  4.266495f,  4.539742f};
	static float xy2[3] =     { -0.998888f, -1.020153f, -1.027479f};
	static float z2[3] =      {  0.857998f,  0.703913f,  0.573625f};
	static float e2[3] =      { 64.208486f, 63.919687f, 43.809866f};

  static Color XY[3] = { Color( xy0[0], xy0[1], xy0[2]), Color( xy1[0], xy1[1], xy1[2]), Color( xy2[0], xy2[1], xy2[2]) };
  static Color  Z[3] = { Color( z0[0], z0[1], z0[2]), Color( z1[0], z1[1], z1[2]), Color( z2[0], z2[1], z2[2]) };
  static Color  E[3] = { Color( e0[0], e0[1], e0[2]), Color( e1[0], e1[1], e1[2]), Color( e2[0], e2[1], e2[2]) };

  m_pBxDFDiffuse = new CBxDFLaFortune( Diffuse, 3, XY, XY, Z, E, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_REFLECTION ) );
}

CMatSkin::~CMatSkin(void)
{
  delete m_pBxDFDiffuse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CMatSkin::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );
  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

Color CMatSkin::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade);

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
}
// vim:ts=2
