////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "texture.h"
#include "perlinnoise.h"
#include "geometry/ray.h"


class CNoiseTexture : public CTextureBase, public CDisplacementTexture
{
public:
	enum Style { cNormal,	cTurbulent, cMarble, cArtistic, cTurbulent2	};

	virtual Vector Value( const double u, const double v, const Vector& p, double Scale ) const;

	void	 SetMinMax( double Min, double Max ) { m_Min = Min, m_Max = Max; };

	// get bump normal based on the current perlin noise configuration & current hit ray
	Vector GetBumpNormal( const Ray* pCurRay, double BumpScaling) override;

	// displace the hit point of the hit ray
	void DisplaceHitRay( Ray* pCurRay, GeometryEngine* pEngine, double BumpScaling ) override;

	// texture coordinate creation - nothing to do for noise textures
	bool	  GetTextureCoordinates( const Ray* pCurRay, double& u, double& v ) const override { UNUSED( pCurRay ); UNUSED( u ); UNUSED( v ); return false;  };

public:
	Style						m_Style = Style::cMarble;

protected:
	double					m_Min = 0.0;
	double					m_Max = 1.0;
	CPerlinNoise		m_NoiseGen;

};
