/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	14-Jun-2013
 */

#include "matrix2.h"

#include <iomanip>

using std::endl;

constexpr double Matrix2::_identity[4];

bool Matrix2::isIdentity() const {
        return !memcmp(data, _identity, sizeof(data));
}

void Matrix2::identity() {
        memcpy(data, _identity, sizeof(data));
}

/** transpose */
void Matrix2::transpose()
{
	double t = M(0,1);
	M(1,0) = M(0,1);
	M(0,1) = t;
} // transpose

/** operator * *
Matrix2& Matrix2::operator *= (const Matrix2& mat)
{
	// FIXME Optimize
	return M = M*mat;
} * operator *= */

/** negate matrix */
void Matrix2::negate()
{
	M(0,0) = -M(0,0);
	M(0,1) = -M(0,1);
	M(1,0) = -M(1,0);
	M(1,1) = -M(1,1);
} // negate

/** invert the matrix in place and
 * @return true on success, false otherwise
 */
bool Matrix2::inverse(const double eps)
{
	double D = det();
	if (Eq0(D, eps)) return false;
	D = 1.0/D;

	double a = M(0,0);
	double b = M(0,1);
	double c = M(1,0);
	double d = M(1,1);

	M(0,0) =  d*D;
	M(0,1) = -b*D;
	M(1,0) = -c*D;
	M(1,1) =  a*D;

	return true;
} // inverse

/** @return inverse matrix */
Matrix2 Matrix2::inv() const
{
	Matrix2 minv = *this;
	minv.inverse();
	return minv;
} // inv

/** rotate matrix around an axis
 * @param angle
 * */
void Matrix2::rotate(const double angle)
{
	double s, c;
	bsincos(angle, &s, &c);

	M(0,0) =  c;
	M(0,1) = -s;
	M(1,0) =  s;
	M(1,1) =  c;
} // rotate

/** matrix print out */
std::ostream& operator << (std::ostream& s, const Matrix2& matrix)
{
	for (int r=0; r<matrix.rows(); r++) {
		if (r==0)
			s << "/";
		else
		if (r+1==matrix.rows())
			s << "\\";
		else
			s << "|";
		for (int c=0; c<matrix.columns(); c++) {
			s << " " << std::setw(16) << std::setprecision(10) << matrix(r,c);
		}
		if (r==0)
			s << " \\" << endl;
		else
		if (r+1==matrix.rows())
			s << " /" << endl;
		else
			s << " |" << endl;
	}
	return s;
} /* operator << */
