/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	20-Oct-2019
 */

#include "bmath.h"
#include "sphere.h"

// ================================== Sphere ===================================
/** grad */
Vector Sphere::grad(const Point& p) const
{
	Vector g = p-center;
	g.normalize();
	return g;
} // grad

/** @return minimum positive distance from all bodies conics */
double Sphere::distance(const Point& p) const
{
	return R - (p-center).length();
} // distance

/** @return intersection distance with line p+v*t */
int Sphere::distance(const Point& p, const Vector v, double t[]) const
{
	Point pn = p - center;
//	assert(Eq(v.length(), 1, SMALL5));
	double b = 2*v*pn;
	double c = pn.length2() - Sq(R);
	int n = quadratic(b, c, &t[0], &t[1], SMALL);
	if (n>=1) {
		if (t[0]>t[1]) Swap(t[0], t[1]);
	} else
		return 0;
	return 2;
} // distance
