/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Usrbin manipulator allowing for calculating ratios, sums etc. of usrbins
 *
 * Authors:	Vasilis.Vlachoudis@cern.ch
 *					Christian.Theis@cern.ch
 *
 * Date:	03-Mar-2023
*/

#ifndef USRBIN_MANIP_H
#define USRBIN_MANIP_H

#include <cstdio>
#include <cassert>

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <utility>
#include <functional>
#include <map>
#include <memory>
#include <numeric>

#include "os.h"
#include "data/usrbin.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// data
//
std::vector<std::unique_ptr<Usrbin> > _meshes;
std::vector<double> _meshWeights;

std::map<std::string, std::function<double( double a, double b, double w1, double w2 ) > > _manipulators =
{
	{ "+", []( double a, double b, double w1, double w2 ) { return (a * w1) + (b * w2); } },
	{ "-", []( double a, double b, double w1, double w2 ) { return (a * w1) - (b * w2); } },
	{ "*", []( double a, double b, double w1, double w2 ) { return (a * w1) * (b * w2); } },
	{ "/", []( double a, double b, double w1, double w2 ) { if ( b == 0.0 ) return 0.0; return (a * w1) / (b * w2); } },
	{ "|-|", []( double a, double b, double w1, double w2 ) { return std::fabs(a - b); } }
};

std::map<std::string, std::function<double( double a, double w1 ) > > _variance =
{
	{ "%", []( double a, double w1 ) { return (a * w1) * (a * w1); } },
	{ "0", []( double a, double w1 ) { return (a * a); } }
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// functions
//
bool LoadMeshes( const std::vector<std::string>& fileNames, int detectorIndex );
bool ManipulateMeshes( const std::string& outputFileName, std::function<double( double a, double b, double w1, double w2 ) > mathOp, std::function<double( double a, double w1 ) > varOp, const std::string& opStr );


#endif
