/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __VIEWER_LAYER_H
#define __VIEWER_LAYER_H

#include "os.h"
#include "geometry/geo.h"

class BBox;
class OBBox;
class Point;
class Color3D;
class ZPainter;
class ViewPort;
class Geometry;
class GeometryViewer;
class GeometryKernel;
class GeometryEngine;

class Layer {
public:
const	Geometry&	geometry;
	GeometryKernel&	kernel;
	GeometryViewer&	viewer;

public:
	int	show;		/** show current layer */

public:
	Layer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);
	ViewPort&	view()	const;
	bool		stop()	const;

	GeometryEngine*	engine() const;
#ifdef THREAD
	GeometryEngine*	engine(int i) const;
#endif

	// 3D Drawing methods
	// OLD PSEUDO 3D line using the Color3D
	bool	draw3Dline(ZPainter& painter, const Point& a, const Point& b,
				const Color3D& color);
	// NEW 3D line through ZPainter
	bool	draw3Dline(ZPainter& painter, const Point& a, const Point& b,
				dword color, int alpha=0);

	void	drawBBox(ZPainter& painter, const BBox& bbox, const Color3D& color);
#ifdef USE_OBBOX
	void	drawOBBox(ZPainter& painter, const OBBox& bbox, const Color3D& color);
#endif
}; // Layer
#endif
